==> divine-4.1.2+2018.02.17.tar.gz: divine-4.1.2+2018.02.17/tools/runtime-cc.cpp <==
#include <divine/cc/clang.hpp>
#include <divine/cc/paths.hpp>

DIVINE_RELAX_WARNINGS
#include <llvm/Bitcode/ReaderWriter.h>
DIVINE_UNRELAX_WARNINGS

#include <lart/divine/debugpaths.hpp>
#include <brick-string>
#include <brick-fs>
#include <brick-llvm>
#include <brick-sha2>
#include <iostream>

using namespace divine;
using namespace brick;

/* usage: runtime-cc srcdir bindir source.c output.bc [flags] */
int main( int argc, const char **argv )
{
    try {
        cc::Compiler clang;
        std::string srcDir = argv[1], binDir = argv[2];
        clang.allowIncludePath( srcDir );
        clang.allowIncludePath( binDir );
        std::vector< std::string > opts;
        std::copy( argv + 5, argv + argc, std::back_inserter( opts ) );

        /* Compile only if SHA-2 hash of preprocessed file differs */
        auto prec = clang.preprocessModule( argv[3], opts );
        std::string fpFilename = std::string( argv[ 4 ] ) + ".fp";
        std::string oldFp = fs::readFileOr( fpFilename, {} );
        std::string newFp = sha2_512( prec );
        if ( oldFp == newFp && fs::exists( argv[ 4 ] ) ) {
            fs::touch( argv[ 4 ] );
            return 0;
        }
        auto mod = clang.compileModule( argv[3], opts );

        auto runtimeBase = fs::joinPath( srcDir, "runtime" );
        if ( !fs::exists( runtimeBase ) )
                runtimeBase = srcDir;
        lart::divine::rewriteDebugPaths( *mod, [=]( auto p ) {
                std::string relpath;
                if ( string::startsWith( p, runtimeBase ) )
                    relpath = p.substr( runtimeBase.size() );
                else if ( string::startsWith( p, binDir ) )
                    relpath = p.substr( binDir.size() );
                if ( !relpath.empty() ) {
                    while ( fs::isPathSeparator( relpath[0] ) )
                        relpath.erase( relpath.begin() );
                    return fs::joinPath( cc::directory( relpath ), relpath );
                }
                return p;
            } );

        brick::llvm::writeModule( mod.get(), argv[4] );
        fs::writeFile( fpFilename, newFp );

        return 0;
    } catch ( cc::CompileError &err ) {
        std::cerr << err.what() << std::endl;
        return 1;
    }
}

==> divine-4.1.5+2018.05.06.tar.gz: divine-4.1.5+2018.05.06/tools/runtime-cc.cpp <==
#include <divine/cc/clang.hpp>
#include <divine/cc/paths.hpp>

DIVINE_RELAX_WARNINGS
#include <llvm/Bitcode/ReaderWriter.h>
DIVINE_UNRELAX_WARNINGS

#include <lart/divine/debugpaths.hpp>
#include <brick-string>
#include <brick-fs>
#include <brick-llvm>
#include <brick-sha2>
#include <iostream>

using namespace divine;
using namespace brick;

/* usage: runtime-cc srcdir bindir source.c output.bc [flags] */
int main( int argc, const char **argv )
{
    try {
        cc::Compiler clang;
        std::string srcDir = argv[1], binDir = argv[2];
        clang.allowIncludePath( srcDir );
        clang.allowIncludePath( binDir );
        std::vector< std::string > opts;
        std::copy( argv + 5, argv + argc, std::back_inserter( opts ) );

        /* Compile only if SHA-2 hash of preprocessed file differs */
        auto prec = clang.preprocessModule( argv[3], opts );
        std::string fpFilename = std::string( argv[ 4 ] ) + ".fp";
        std::string oldFp = fs::readFileOr( fpFilename, {} );
        std::string newFp = sha2_512( prec );
        if ( oldFp == newFp && fs::exists( argv[ 4 ] ) ) {
            fs::touch( argv[ 4 ] );
            return 0;
        }
        auto mod = clang.compileModule( argv[3], opts );

        auto runtimeBase = fs::joinPath( srcDir, "dios" );
        if ( !fs::exists( runtimeBase ) )
                runtimeBase = srcDir;
        lart::divine::rewriteDebugPaths( *mod, [=]( auto p ) {
                std::string relpath;
                if ( string::startsWith( p, runtimeBase ) )
                    relpath = p.substr( runtimeBase.size() );
                else if ( string::startsWith( p, binDir ) )
                    relpath = p.substr( binDir.size() );
                if ( !relpath.empty() ) {
                    while ( fs::isPathSeparator( relpath[0] ) )
                        relpath.erase( relpath.begin() );
                    return fs::joinPath( cc::directory( relpath ), relpath );
                }
                return p;
            } );

        brick::llvm::writeModule( mod.get(), argv[4] );
        fs::writeFile( fpFilename, newFp );

        return 0;
    } catch ( cc::CompileError &err ) {
        std::cerr << err.what() << std::endl;
        return 1;
    }
}
